#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jun  3 10:41:29 2020

@author: Maximilian Schlosshauer, University of Portland
"""

from scipy.stats import ks_2samp
from scipy import stats
import numpy as np
import matplotlib as mpl 
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import os 

# SS_test data 
import borel 
import test1 
import test1_comp 
import test2 
import test2_comp 
import test3
import test3_comp 
import test4 
import test4_comp 

os.environ["PATH"] += os.pathsep + '/usr/local/texlive/2016/bin/universal-darwin'
plt.rcParams['text.usetex'] = True

# To input a newline character in query-replace, use C-q C-j.

def make_boxplot(data_to_plot,include_parity,outname,specy,ymin,ymax) :
    fig = plt.figure(1, figsize=(9, 6))
    ax = fig.add_subplot(111)
    
    meanpointprops = dict(marker='D', markeredgecolor='black',
                          markerfacecolor='firebrick')
    
    bp = ax.boxplot(data_to_plot, meanprops=meanpointprops, showmeans=True)
    
    bp = ax.boxplot(data_to_plot, patch_artist=True)
    
    ## change outline color, fill color and linewidth of the boxes
    i=0
    if (include_parity == 0) :
        colors = ['lightblue','palegreen','gold','steelblue','teal','orange']
    if (include_parity == 1) :
        colors = ['lightblue','silver','palegreen','gold','steelblue','teal','orange']
    if (include_parity == 2) :
        colors = ['lightblue','palegreen','gold','steelblue','teal','orange']
    for box in bp['boxes']:
        # change outline color
        box.set( color='#7570b3', linewidth=2)
        # change fill color
        box.set( facecolor = colors[i])
        i=i+1
#        box.set( facecolor = '#1b9e77')
    
    ## change color and linewidth of the whiskers
    for whisker in bp['whiskers']:
        whisker.set(color='#7570b3', linewidth=2)

    plt.xticks(size = 13)
    plt.yticks(size = 13)
    if (specy==1) :
        axes = plt.gca()
        axes.set_ylim([ymin,ymax])
    
    ## change color and linewidth of the caps
    for cap in bp['caps']:
        cap.set(color='#7570b3', linewidth=2)
    
    ## change color and linewidth of the medians
    for median in bp['medians']:
        median.set(color='#b2df8a', linewidth=2)
    
    ## change the style of fliers and their fill
    for flier in bp['fliers']:
        flier.set(marker='o', color='#e7298a', alpha=0.5)
    
    ## Custom x-axis labels
    if (include_parity == 0) :
        ax.set_xticklabels(['$\mathrm{ANU\;QRNG}$', '$\mathrm{AES\mbox{-}OFB}$', '$\mathrm{Gfsr4}$', '$\mathrm{Mt19937}$', '$\mathrm{Threefish}$'])
    if (include_parity == 1) :
        ax.set_xticklabels(['$\mathrm{ANU\;QRNG}$', '$\mathrm{Parity\;QRNG}$', '$\mathrm{AES\mbox{-}OFB}$', '$\mathrm{Gfsr4}$', '$\mathrm{Mt19937}$', '$\mathrm{Threefish}$'])
    if (include_parity == 2) :
        ax.set_xticklabels(['KS (with)', 'KS (without)', 
                            'Welch (with)', 'Welch (without)'])
    
    ## Remove top axes and right axes ticks
    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()
                  
    # Save the figure
    fig.show()
    
    fig.savefig(outname, bbox_inches='tight')



# ******************************************************************************************
# Choose two data sets
# ******************************************************************************************


anu_borel = borel.anu_borel[0:100]
parity_borel = borel.parity_borel[0:100]
aes_ofb_borel = borel.aes_ofb_borel[0:100]
gfsr4_borel = borel.gfsr4_borel[0:100]
mt19937_borel = borel.mt19937_borel[0:100]
threefish_borel = borel.threefish_borel[0:100]

anu_1 = test1.anu_1[0:100]
parity_1 = test1.parity_1[0:100]
aes_ofb_1 = test1.aes_ofb_1[0:100]
gfsr4_1 = test1.gfsr4_1[0:100]
mt19937_1 = test1.mt19937_1[0:100]
threefish_1 = test1.threefish_1[0:100]

anu_1_comp = test1_comp.anu_1_comp[0:100]
parity_1_comp = test1_comp.parity_1_comp[0:100]
aes_ofb_1_comp = test1_comp.aes_ofb_1_comp[0:100]
gfsr4_1_comp = test1_comp.gfsr4_1_comp[0:100]
mt19937_1_comp = test1_comp.mt19937_1_comp[0:100]
threefish_1_comp = test1_comp.threefish_1_comp[0:100]

anu_2 = test2.anu_2[0:100]
parity_2 = test2.parity_2[0:100]
aes_ofb_2 = test2.aes_ofb_2[0:100]
gfsr4_2 = test2.gfsr4_2[0:100]
mt19937_2 = test2.mt19937_2[0:100]
threefish_2 = test2.threefish_2[0:100]

anu_2_comp = test2_comp.anu_2_comp[0:100]
parity_2_comp = test2_comp.parity_2_comp[0:100]
aes_ofb_2_comp = test2_comp.aes_ofb_2_comp[0:100]
gfsr4_2_comp = test2_comp.gfsr4_2_comp[0:100]
mt19937_2_comp = test2_comp.mt19937_2_comp[0:100]
threefish_2_comp = test2_comp.threefish_2_comp[0:100]

anu_3 = test3.anu_3[0:100]
parity_3 = test3.parity_3[0:100]
aes_ofb_3 = test3.aes_ofb_3[0:100]
gfsr4_3 = test3.gfsr4_3[0:100]
mt19937_3 = test3.mt19937_3[0:100]
threefish_3 = test3.threefish_3[0:100]

anu_3_comp = test3_comp.anu_3_comp[0:100]
parity_3_comp = test3_comp.parity_3_comp[0:100]
aes_ofb_3_comp = test3_comp.aes_ofb_3_comp[0:100]
gfsr4_3_comp = test3_comp.gfsr4_3_comp[0:100]
mt19937_3_comp = test3_comp.mt19937_3_comp[0:100]
threefish_3_comp = test3_comp.threefish_3_comp[0:100]

anu_4 = test4.anu_4[0:100]
parity_4 = test4.parity_4[0:100]
aes_ofb_4 = test4.aes_ofb_4[0:100]
gfsr4_4 = test4.gfsr4_4[0:100]
mt19937_4 = test4.mt19937_4[0:100]
threefish_4 = test4.threefish_4[0:100]

anu_4_comp = test4_comp.anu_4_comp[0:100]
parity_4_comp = test4_comp.parity_4_comp[0:100]
aes_ofb_4_comp = test4_comp.aes_ofb_4_comp[0:100]
gfsr4_4_comp = test4_comp.gfsr4_4_comp[0:100]
mt19937_4_comp = test4_comp.mt19937_4_comp[0:100]
threefish_4_comp = test4_comp.threefish_4_comp[0:100]

number_of_sets = 8

# ******************************************************************************************
# Kolmogorov–Smirnov test for two samples
# ******************************************************************************************

print "\nKolmogorov–Smirnov test for two samples:"

datasets_names_borel = ['anu_borel', 'parity_borel', 'aes_ofb_borel', 'gfsr4_borel', 'mt19937_borel', 'threefish_borel']
datasets_names_1 = ['anu_1', 'anu_1_comp', 'parity_1', 'parity_1_comp', 'aes_ofb_1', 'gfsr4_1', 'mt19937_1', 'threefish_1']
datasets_names_2 = ['anu_2', 'anu_2_comp', 'parity_2', 'parity_2_comp', 'aes_ofb_2', 'gfsr4_2', 'mt19937_2', 'threefish_2']
datasets_names_3 = ['anu_3', 'anu_3_comp', 'parity_3', 'parity_3_comp', 'aes_ofb_3', 'gfsr4_3', 'mt19937_3', 'threefish_3']
datasets_names_4 = ['anu_4', 'anu_4_comp', 'parity_4', 'parity_4_comp', 'aes_ofb_4', 'gfsr4_4', 'mt19937_4', 'threefish_4']
datasets_names_1_comp = ['anu_1', 'anu_1_comp', 'parity_1', 'parity_1_comp', 'aes_ofb_1_comp', 'gfsr4_1_comp', 'mt19937_1_comp', 'threefish_1_comp']
datasets_names_2_comp = ['anu_2', 'anu_2_comp', 'parity_2', 'parity_2_comp', 'aes_ofb_2_comp', 'gfsr4_2_comp', 'mt19937_2_comp', 'threefish_2_comp']
datasets_names_3_comp = ['anu_3', 'anu_3_comp', 'parity_3', 'parity_3_comp', 'aes_ofb_3_comp', 'gfsr4_3_comp', 'mt19937_3_comp', 'threefish_3_comp']
datasets_names_4_comp = ['anu_4', 'anu_4_comp', 'parity_4', 'parity_4_comp', 'aes_ofb_4_comp', 'gfsr4_4_comp', 'mt19937_4_comp', 'threefish_4_comp']

datasets_borel = [anu_borel, parity_borel, aes_ofb_borel, gfsr4_borel, mt19937_borel, threefish_borel]
datasets_1 = [anu_1, anu_1_comp, parity_1, parity_1_comp, aes_ofb_1, gfsr4_1, mt19937_1, threefish_1]
datasets_2 = [anu_2, anu_2_comp, parity_2, parity_2_comp, aes_ofb_2, gfsr4_2, mt19937_2, threefish_2]
datasets_3 = [anu_3, anu_3_comp, parity_3, parity_3_comp, aes_ofb_3, gfsr4_3, mt19937_3, threefish_3]
datasets_4 = [anu_4, anu_4_comp, parity_4, parity_4_comp, aes_ofb_4, gfsr4_4, mt19937_4, threefish_4]
datasets_1_comp = [anu_1, anu_1_comp, parity_1, parity_1_comp, aes_ofb_1_comp, gfsr4_1_comp, mt19937_1_comp, threefish_1_comp]
datasets_2_comp = [anu_2, anu_2_comp, parity_2, parity_2_comp, aes_ofb_2_comp, gfsr4_2_comp, mt19937_2_comp, threefish_2_comp]
datasets_3_comp = [anu_3, anu_3_comp, parity_3, parity_3_comp, aes_ofb_3_comp, gfsr4_3_comp, mt19937_3_comp, threefish_3_comp]
datasets_4_comp = [anu_4, anu_4_comp, parity_4, parity_4_comp, aes_ofb_4_comp, gfsr4_4_comp, mt19937_4_comp, threefish_4_comp]


datasets_names = datasets_names_4  # CHOOSE
datasets = datasets_4;    # CHOOSE

for i in range(0,number_of_sets-1,1) :
    for j in range(i+1,number_of_sets,1) :
        print "(" + datasets_names[i] + ", " + datasets_names[j] + ") :", ks_2samp(datasets[i], datasets[j])


# If the KS statistic is small or the p-value is high (bigger than the threshold we set), then we cannot reject 
# the hypothesis that the distributions of the two samples are the same.
# see: https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.ks_2samp.html

# From Abbott: This test returns a p-value indicating the probability, given the observed 
# test statistic, that the observed distributions were indeed drawn from the same 
# distribution. We conclude that ‘the difference between the two datasets is 
# statistically significant’ if the p-value is less than 0.005. 

# Hodges, J.L. Jr., “The Significance Probability of the Smirnov Two-Sample Test,” Arkiv fiur Matematik, 3, No. 43 (1958), 469-86.

# ******************************************************************************************
# Shapiro-Wilk: checks whether the test metric distribution is consistent with a normal distribution 
# See: https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.shapiro.html
# Abbott: Here we consider evidence for non-normality to be a p-value below 0.05.
# ******************************************************************************************

print "\nShapiro-Wilk:"
for i in range(0,number_of_sets,1) :
    print datasets_names_2[i] + " : ", stats.shapiro(datasets[i])  # second output value is the p-value


# Welch’s t-test
print "\nWelch’s t-test:"

for i in range(0,number_of_sets-1,1) :
    for j in range(i+1,number_of_sets,1) :
        print "(" + datasets_names[i] + ", " + datasets_names[j] + ") : ", stats.ttest_ind(datasets[i], datasets[j],  equal_var = False)

# Make boxplot
data_to_plot_borel = [anu_borel, parity_borel, aes_ofb_borel, gfsr4_borel, mt19937_borel, threefish_borel]
data_to_plot_1 = [anu_1, parity_1, aes_ofb_1, gfsr4_1, mt19937_1, threefish_1]
data_to_plot_2 = [anu_2, parity_2, aes_ofb_2, gfsr4_2, mt19937_2, threefish_2]
data_to_plot_3 = [anu_3, parity_3, aes_ofb_3, gfsr4_3, mt19937_3, threefish_3]
data_to_plot_4 = [anu_4, aes_ofb_4, gfsr4_4, mt19937_4, threefish_4]
data_to_plot_1_comp = [anu_1_comp, parity_1_comp, aes_ofb_1_comp, gfsr4_1_comp, mt19937_1_comp, threefish_1_comp]
data_to_plot_2_comp = [anu_2_comp, parity_2_comp, aes_ofb_2_comp, gfsr4_2_comp, mt19937_2_comp, threefish_2_comp]
data_to_plot_3_comp = [anu_3_comp, parity_3_comp, aes_ofb_3_comp, gfsr4_3_comp, mt19937_3_comp, threefish_3_comp]
data_to_plot_4_comp = [anu_4_comp, aes_ofb_4_comp, gfsr4_4_comp, mt19937_4_comp, threefish_4_comp]

#make_boxplot(data_to_plot_borel, 1, 'borel.pdf', 0, 17, 34)    # CHOOSE

#make_boxplot(data_to_plot_1, 1, 'test1.pdf', 1, 17, 34)    # CHOOSE
#make_boxplot(data_to_plot_1_comp, 1, 'test1_comp.pdf', 1, 17, 34)    # CHOOSE

#make_boxplot(data_to_plot_2, 1, 'test2.pdf', 1, 7.8650e8, 7.8850e8)    # CHOOSE
make_boxplot(data_to_plot_2_comp, 1, 'test2_comp.pdf', 1, 7.8650e8, 7.8850e8)    # CHOOSE

#make_boxplot(data_to_plot_3, 1, 'test3.pdf', 1, 7.870e8, 7.884e8)    # CHOOSE
#make_boxplot(data_to_plot_3_comp, 1, 'test3_comp.pdf', 1, 7.870e8, 7.884e8)    # CHOOSE

#make_boxplot(data_to_plot_4, 0, 'test4.pdf', 1, 70, 180)    # CHOOSE
#make_boxplot(data_to_plot_4_comp, 0, 'test4_comp.pdf', 1, 70, 180)    # CHOOSE
